// ignore_for_file: avoid_print
// ignore_for_file: public_member_api_docs

import 'package:flutter/material.dart';

import 'package:flutter_sticky_grouped_list_view/sticky_grouped_list_view.dart';

void main() => runApp(App());

class App extends StatelessWidget {
  @override
  Widget build(BuildContext context) => MaterialApp(
        title: 'Flutter Sticky Grouped List View Demo',
        theme: ThemeData(
          primarySwatch: Colors.blueGrey,
          visualDensity: VisualDensity.adaptivePlatformDensity,
        ),
        home: const HomePage(title: 'Sticky Grouped List View'),
      );
}

class HomePage extends StatefulWidget {
  const HomePage({Key key, this.title}) : super(key: key);

  final String title;

  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  @override
  Widget build(BuildContext context) => Scaffold(
        appBar: AppBar(
          elevation: 0,
          title: Text(widget.title),
        ),
        body: _getGroupedListView(),
      );
}

StickyGroupedListView<dynamic, String> _getGroupedListView() =>
    StickyGroupedListView<dynamic, String>(
      elements: _elements,
      hasStickyHeader: true,
      hasRefreshIndicator: true,
      refreshIndicatorDisplacement: 60,
      groupBy: (dynamic element) => element['group'],
      groupBuilder: (String value) => _buildGroup(value),
      itemBuilder: (BuildContext context, dynamic element) =>
          _buildItem(element),
      separator: _buildSeparator(),
      onRefresh: () async {
        await Future<void>.delayed(
          const Duration(seconds: 2),
          () => print('refresh completed.'),
        );
      },
    );

Widget _buildGroup(String value) => Container(
      color: Colors.blueGrey[100],
      child: ListTile(
        title: Text(
          value,
          style: const TextStyle(
            fontSize: 16,
            fontWeight: FontWeight.bold,
          ),
        ),
      ),
    );

Widget _buildItem(dynamic element) => ListTile(
      leading: _buildLeading(),
      title: _buildTitle(element),
      subtitle: _buildSubtitle(element),
      trailing: _buildTrailing(),
    );

Widget _buildLeading() => const CircleAvatar(
      child: Icon(Icons.accessibility_new),
      radius: 18,
    );

Widget _buildTitle(dynamic element) => Text(
      element['title'],
      style: const TextStyle(
        fontSize: 15,
        fontWeight: FontWeight.normal,
      ),
    );

Widget _buildSubtitle(dynamic element) => Text(
      element['subtitle'],
      style: const TextStyle(
        fontSize: 13,
        fontWeight: FontWeight.normal,
      ),
    );

Widget _buildTrailing() => IconButton(
      icon: const Icon(Icons.navigate_next),
      onPressed: () {},
    );

Widget _buildSeparator() => Container(
      color: Colors.blueGrey[100],
      height: 1,
    );

final List<Map<String, String>> _elements = <Map<String, String>>[
  <String, String>{
    'title': 'Title 01',
    'subtitle': 'Subtitle 01',
    'group': 'Group 01'
  },
  <String, String>{
    'title': 'Title 02',
    'subtitle': 'Subtitle 02',
    'group': 'Group 01'
  },
  <String, String>{
    'title': 'Title 03',
    'subtitle': 'Subtitle 03',
    'group': 'Group 02'
  },
  <String, String>{
    'title': 'Title 04',
    'subtitle': 'Subtitle 04',
    'group': 'Group 02'
  },
  <String, String>{
    'title': 'Title 05',
    'subtitle': 'Subtitle 05',
    'group': 'Group 02'
  },
  <String, String>{
    'title': 'Title 06',
    'subtitle': 'Subtitle 06',
    'group': 'Group 03'
  },
  <String, String>{
    'title': 'Title 07',
    'subtitle': 'Subtitle 07',
    'group': 'Group 03'
  },
  <String, String>{
    'title': 'Title 08',
    'subtitle': 'Subtitle 08',
    'group': 'Group 03'
  },
  <String, String>{
    'title': 'Title 09',
    'subtitle': 'Subtitle 09',
    'group': 'Group 03'
  },
  <String, String>{
    'title': 'title 10',
    'subtitle': 'Subtitle 10',
    'group': 'Group 03'
  },
  <String, String>{
    'title': 'title 11',
    'subtitle': 'Subtitle 11',
    'group': 'Group 04'
  },
  <String, String>{
    'title': 'title 12',
    'subtitle': 'Subtitle 12',
    'group': 'Group 04'
  },
  <String, String>{
    'title': 'title 13',
    'subtitle': 'Subtitle 13',
    'group': 'Group 04'
  },
  <String, String>{
    'title': 'title 14',
    'subtitle': 'Subtitle 14',
    'group': 'Group 04'
  },
  <String, String>{
    'title': 'title 15',
    'subtitle': 'Subtitle 15',
    'group': 'Group 04'
  },
  <String, String>{
    'title': 'title 16',
    'subtitle': 'Subtitle 16',
    'group': 'Group 04'
  },
  <String, String>{
    'title': 'title 17',
    'subtitle': 'Subtitle 17',
    'group': 'Group 05'
  },
  <String, String>{
    'title': 'title 18',
    'subtitle': 'Subtitle 18',
    'group': 'Group 05'
  },
  <String, String>{
    'title': 'title 19',
    'subtitle': 'Subtitle 19',
    'group': 'Group 05'
  },
  <String, String>{
    'title': 'title 20',
    'subtitle': 'Subtitle 20',
    'group': 'Group 05'
  },
];
